#include "fss_read.h"
#include "../main/fss_read.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_fss_read_embedded_list_process_
  void fss_read_embedded_list_process(void * const void_main) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    // The process_load() callback is required to be non-NULL.
    if (!main->callback.process_load) return;

    // This standard does not support multiple select, so any select greater than 0 can be predicted without processing the buffer.
    if ((main->setting.flag & fss_read_main_flag_select_d) && main->setting.select) {
      if (main->setting.flag & fss_read_main_flag_total_d) {
        fss_read_print_number(&main->program.output, 0);
      }

      main->setting.state.status = F_okay;

      return;
    }

    if (main->setting.buffer.used) {
      main->setting.range.start = 0;
      main->setting.range.stop = main->setting.buffer.used;
    }
    else {
      main->setting.range.start = 1;
      main->setting.range.stop = 0;
    }

    main->setting.objects.used = 0;
    main->setting.contents.used = 0;
    main->setting.comments.used = 0;
    main->setting.delimits_object.used = 0;
    main->setting.delimits_content.used = 0;
    main->setting.nest.used = 0;
    main->setting.quotes_object.used = 0;
    main->setting.quotes_content.used = 0;

    main->callback.process_load(main);
    if (F_status_is_error(main->setting.state.status)) return;

    fss_read_embedded_list_process_determine_depth(main);
    if (F_status_is_error(main->setting.state.status)) return;

    const f_number_unsigneds_t * const delimits_object = !(main->setting.flag & fss_read_main_flag_original_d) && fss_read_delimit_object_is(main, 0) ? &main->setting.delimits_object : &fss_read_except_none_c;
    const f_number_unsigneds_t * const delimits_content = !(main->setting.flag & fss_read_main_flag_original_d) && fss_read_delimit_content_is(main, 0) ? &main->setting.delimits_content : &fss_read_except_none_c;

    bool names[main->setting.objects.used];

    if (main->callback.process_name) {
      main->callback.process_name(main, names);
      if (F_status_is_error(main->setting.state.status)) return;
    }
    else if (main->setting.objects.used) {

      // If no processing is enabled, then default to F_true to enable all names.
      memset(names, F_true, sizeof(bool) * main->setting.objects.used);
    }

    if (main->setting.flag & fss_read_main_flag_total_d) {
      if (main->callback.process_total) {
        main->callback.process_total(main, names);
      }
    }
    else if ((main->setting.flag & fss_read_main_flag_at_d) && main->setting.depth.index_at) {
      if (main->callback.process_at) {
        main->callback.process_at(main, names, *delimits_object, *delimits_content);
      }
    }
    else if (main->setting.flag & fss_read_main_flag_columns_d) {
      if (main->callback.process_columns) {
        main->callback.process_columns(main, names);
      }
    }
    else {
      f_number_unsigned_t i = 0;

      if (main->setting.flag & fss_read_main_flag_line_d) {
        if (main->callback.process_at_line) {
          f_number_unsigned_t line = 0;

          for (; i < main->setting.contents.used && main->setting.state.status != F_success; ++i) {

            if (!names[i]) continue;
            if (fss_read_signal_check(main)) return;

            main->callback.process_at_line(void_main, i, *delimits_object, *delimits_content, &line);
            if (F_status_is_error(main->setting.state.status)) return;
          } // for
        }
      }
      else if (main->callback.print_at) {
        for (; i < main->setting.contents.used; ++i) {

          if (!names[i]) continue;
          if (fss_read_signal_check(main)) return;

          main->callback.print_at(&main->program.output, i, *delimits_object, *delimits_content);
        } // for
      }
    }

    if (F_status_is_error(main->setting.state.status)) return;

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_embedded_list_process_

#ifndef _di_fss_read_embedded_list_process_determine_depth_
  void fss_read_embedded_list_process_determine_depth(fss_read_main_t * const main) {

    if (!main || !main->setting.nest.used || !main->setting.nest.depth[0].used || !main->setting.depths.used) return;

    if (main->setting.depths.array[0].depth >= main->setting.nest.used) {
      main->setting.state.status = F_status_set_error(F_parameter);

      return;
    }

    if ((main->setting.flag & fss_read_main_flag_depth_d) && (main->setting.depths.array[0].depth || main->setting.depths.used > 1) && main->setting.nest.used > 1) {
      f_number_unsigned_t i = 0;

      // Pre-process depths to confirm if any requested i is out of range.
      for (; i < main->setting.depths.used; ++i) {

        // Out of range is not an error, there is simply nothing to print.
        if (main->setting.depths.array[i].depth >= main->setting.nest.used) {
          main->setting.state.status = F_okay;

          return;
        }
      } // for

      {
        f_number_unsigned_t at = 0;
        fss_read_depth_t * const the = &main->setting.depths.array[0];

        for (i = 0; i < main->setting.nest.depth[the->depth].used; ++i) {

          if (the->index_name) {
            if (f_compare_dynamic_partial_trim_string(the->value_name.string, main->setting.buffer, the->value_name.used, main->setting.nest.depth[the->depth].array[i].object) != F_equal_to) continue;
          }

          if (the->index_at) {
            if (at++ != the->value_at) {
              if (at > the->value_at) break;

              continue;
            }
          }

          if (main->setting.depths.used > 1) {
            fss_read_embedded_list_process_determine_depth_nest_recurse(main, i, 1);
            if (F_status_is_error(main->setting.state.status)) break;
          }
          else {
            fss_read_embedded_list_process_determine_depth_save(main, &main->setting.nest.depth[the->depth].array[i]);
            if (F_status_is_error(main->setting.state.status)) break;
          }
        } // for

        if (F_status_is_error_not(main->setting.state.status)) {
          fss_read_ensure_quotes_length(main);

          if (F_status_is_error_not(main->setting.state.status)) {
            main->setting.state.status = F_okay;
          }
        }
      }

      if (main->setting.depths.used > 1) {
        main->setting.depth = main->setting.depths.array[main->setting.depths.used - 1];
      }

      return;
    }

    {
      const f_number_unsigned_t depth = main->setting.depths.array[0].depth;

      main->setting.state.status = f_memory_array_resize(main->setting.nest.depth[depth].used, sizeof(f_range_t), (void **) &main->setting.objects.array, &main->setting.objects.used, &main->setting.objects.size);

      if (F_status_is_error_not(main->setting.state.status)) {
        main->setting.state.status = f_memory_array_resize(main->setting.nest.depth[depth].used, sizeof(f_range_t), (void **) &main->setting.closes.array, &main->setting.closes.used, &main->setting.closes.size);
      }

      if (F_status_is_error_not(main->setting.state.status)) {
        main->setting.state.status = f_memory_array_resize(main->setting.nest.depth[depth].used, sizeof(f_ranges_t), (void **) &main->setting.contents.array, &main->setting.contents.used, &main->setting.contents.size);
      }

      if (F_status_is_error(main->setting.state.status)) return;

      for (; main->setting.objects.used < main->setting.nest.depth[depth].used; ++main->setting.objects.used) {

        main->setting.objects.array[main->setting.objects.used] = main->setting.nest.depth[depth].array[main->setting.objects.used].object;
        main->setting.closes.array[main->setting.objects.used] = main->setting.nest.depth[depth].array[main->setting.objects.used].close;

        // Use a static array for the inner Content that points to the depth, and so ensure the size is 0 to designate this is not dynamically allocated here.
        if (main->setting.nest.depth[depth].array[main->setting.objects.used].content.used) {
          main->setting.contents.array[main->setting.objects.used].array = main->setting.nest.depth[depth].array[main->setting.objects.used].content.array;
          main->setting.contents.array[main->setting.objects.used].used = main->setting.nest.depth[depth].array[main->setting.objects.used].content.used;
          main->setting.contents.array[main->setting.objects.used].size = 0;
        }
        else {
          main->setting.contents.array[main->setting.objects.used].array = 0;
          main->setting.contents.array[main->setting.objects.used].used = 0;
          main->setting.contents.array[main->setting.objects.used].size = 0;
        }
      } // for

      main->setting.closes.used = main->setting.objects.used;
      main->setting.contents.used = main->setting.objects.used;
    }

    fss_read_ensure_quotes_length(main);

    if (F_status_is_error_not(main->setting.state.status)) {
      main->setting.state.status = F_okay;
    }
  }
#endif // _di_fss_read_embedded_list_process_determine_depth_

#ifndef _di_fss_read_embedded_list_process_determine_depth_nest_recurse_
  void fss_read_embedded_list_process_determine_depth_nest_recurse(fss_read_main_t * const main, const f_number_unsigned_t parent, const f_number_unsigned_t next) {

    if (!main || !next || next >= main->setting.depths.used) return;

    f_number_unsigned_t i = 0;
    f_number_unsigned_t at = 0;
    fss_read_depth_t * const the = &main->setting.depths.array[next];

    for (; F_status_is_error_not(main->setting.state.status) && i < main->setting.nest.depth[the->depth].used; ++i) {

      if (main->setting.nest.depth[the->depth].array[i].parent != parent) continue;

      if (the->index_name) {
        if (f_compare_dynamic_partial_trim_string(the->value_name.string, main->setting.buffer, the->value_name.used, main->setting.nest.depth[the->depth].array[i].object) != F_equal_to) continue;
      }

      if (the->index_at) {
        if (at++ != the->value_at) {
          if (at > the->value_at) break;

          continue;
        }
      }

      if (next + 1 < main->setting.depths.used) {
        fss_read_embedded_list_process_determine_depth_nest_recurse(main, i, next + 1);
      }
      else {
        fss_read_embedded_list_process_determine_depth_save(main, &main->setting.nest.depth[the->depth].array[i]);
      }
    } // for
  }
#endif // _di_fss_read_embedded_list_process_determine_depth_nest_recurse_

#ifndef _di_fss_read_embedded_list_process_determine_depth_save_
  void fss_read_embedded_list_process_determine_depth_save(fss_read_main_t * const main, f_fss_item_t * const item) {

    if (!main || !item) return;

    main->setting.state.status = f_memory_array_increase(main->setting.state.step_small, sizeof(f_range_t), (void **) &main->setting.objects.array, &main->setting.objects.used, &main->setting.objects.size);

    if (F_status_is_error_not(main->setting.state.status)) {
      main->setting.state.status = f_memory_array_increase(main->setting.state.step_small, sizeof(f_range_t), (void **) &main->setting.closes.array, &main->setting.closes.used, &main->setting.closes.size);
    }

    if (F_status_is_error_not(main->setting.state.status)) {
      main->setting.state.status = f_memory_array_increase(main->setting.state.step_small, sizeof(f_ranges_t), (void **) &main->setting.contents.array, &main->setting.contents.used, &main->setting.contents.size);
    }

    if (F_status_is_error_not(main->setting.state.status)) {
      main->setting.objects.array[main->setting.objects.used] = item->object;
      main->setting.closes.array[main->setting.closes.used] = item->close;

      // Use a static array for the inner Content that points to the depth, and so ensure the size is 0 to designate this is not dynamically allocated here.
      if (item->content.used) {
        main->setting.contents.array[main->setting.contents.used].array = item->content.array;
        main->setting.contents.array[main->setting.contents.used].used = item->content.used;
        main->setting.contents.array[main->setting.contents.used].size = 0;
      }
      else {
        main->setting.contents.array[main->setting.contents.used].array = 0;
        main->setting.contents.array[main->setting.contents.used].used = 0;
        main->setting.contents.array[main->setting.contents.used].size = 0;
      }

      ++main->setting.objects.used;
      ++main->setting.closes.used;
      ++main->setting.contents.used;
    }
  }
#endif // _di_fss_read_embedded_list_process_determine_depth_save_

#ifdef __cplusplus
} // extern "C"
#endif
