/**
 * FLL - Level 3
 *
 * Project: FSS Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides process normal functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _fss_read_embedded_list_process_h
#define _fss_read_embedded_list_process_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Process normally, reading from the input for the Embedded List FSS format.
 *
 * @param main
 *   The program and settings data.
 *
 *   Must not be NULL.
 *   Must be of type fss_read_main_t.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: fss_read_signal_check().
 *
 * @see fss_read_signal_check()
 */
#ifndef _di_fss_read_embedded_list_process_
  extern void fss_read_embedded_list_process(void * const main);
#endif // _di_fss_read_embedded_list_process_

/**
 * Determine the depth in which the processing will happen and construct the Objects and Contents mapping based on this.
 *
 * @param main
 *   The program and settings data.
 *
 *   Must not be NULL.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: fss_read_signal_check().
 *
 * @see fss_read_signal_check()
 */
#ifndef _di_fss_read_embedded_list_process_determine_depth_
  extern void fss_read_embedded_list_process_determine_depth(fss_read_main_t * const main);
#endif // _di_fss_read_embedded_list_process_determine_depth_

/**
 * Recursively determine the depth.
 *
 * This is intended to be directly called by fss_read_embedded_list_process_determine_depth().
 *
 * @param main
 *   The program and settings data.
 *
 *   Must not be NULL.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: fss_read_signal_check().
 * @param parent
 *   The index position of the parent element.
 *   This is needed because the items array for some depth is tied to the parent via a property.
 * @param next
 *   The next index position in the depths array to process.
 *
 *   Must be greater than 0.
 *
 * @see fss_read_signal_check()
 */
#ifndef _di_fss_read_embedded_list_process_determine_depth_nest_recurse_
  extern void fss_read_embedded_list_process_determine_depth_nest_recurse(fss_read_main_t * const main, const f_number_unsigned_t parent, const f_number_unsigned_t next);
#endif // _di_fss_read_embedded_list_process_determine_depth_nest_recurse_

/**
 * Save the given item, appending the parts to the appropriate settings arrays.
 *
 * This is intended to be directly called by fss_read_embedded_list_process_determine_depth() and fss_read_embedded_list_process_determine_depth_nest_recurse().
 *
 * @param main
 *   The program and settings data.
 *
 *   Must not be NULL.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: f_memory_array_increase().
 *
 * @param item
 *   The item to save.
 *
 *   Must not be NULL.
 *
 * @see f_memory_array_increase()
 *
 * @see fss_read_embedded_list_process_determine_depth()
 * @see fss_read_embedded_list_process_determine_depth_nest_recurse()
 */
#ifndef _di_fss_read_embedded_list_process_determine_depth_save_
  extern void fss_read_embedded_list_process_determine_depth_save(fss_read_main_t * const main, f_fss_item_t * const item);
#endif // _di_fss_read_embedded_list_process_determine_depth_save_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fss_read_embedded_list_process_h
