/**
 * FLL - Level 3
 *
 * Project: FSS Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common enumeration structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _fss_read_common_enumeration_h
#define _fss_read_common_enumeration_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The main program parameters.
 */
#ifndef _di_fss_read_parameter_e_
  enum {
    fss_read_parameter_as_e = f_console_standard_parameter_last_e,
    fss_read_parameter_at_e,
    fss_read_parameter_content_e,
    fss_read_parameter_columns_e,
    fss_read_parameter_delimit_e,
    fss_read_parameter_depth_e,
    fss_read_parameter_empty_e,
    fss_read_parameter_line_e,
    fss_read_parameter_name_e,
    fss_read_parameter_object_e,
    fss_read_parameter_payload_e,
    fss_read_parameter_pipe_e,
    fss_read_parameter_original_e,
    fss_read_parameter_select_e,
    fss_read_parameter_total_e,
    fss_read_parameter_trim_e,
  }; // enum

  #define fss_read_console_parameter_t_initialize \
    { \
      macro_fll_program_console_parameter_standard_initialize, \
      \
      macro_f_console_parameter_t_initialize_3(fss_read_short_as_s,       fss_read_long_as_s,       1, f_console_flag_normal_e | f_console_flag_disable_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_at_s,       fss_read_long_at_s,       1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_content_s,  fss_read_long_content_s,  0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_columns_s,  fss_read_long_columns_s,  0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_delimit_s,  fss_read_long_delimit_s,  1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_depth_s,    fss_read_long_depth_s,    1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_empty_s,    fss_read_long_empty_s,    0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_line_s,     fss_read_long_line_s,     1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_name_s,     fss_read_long_name_s,     1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_object_s,   fss_read_long_object_s,   0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_5(                           fss_read_long_payload_s,  1, f_console_flag_normal_e | f_console_flag_disable_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_pipe_s,     fss_read_long_pipe_s,     0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_original_s, fss_read_long_original_s, 0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_select_s,   fss_read_long_select_s,   1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_total_s,    fss_read_long_total_s,    0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(fss_read_short_trim_s,     fss_read_long_trim_s,     0, f_console_flag_normal_e), \
    }

  #define fss_read_parameter_total_d (f_console_parameter_state_type_total_d + 16)
#endif // _di_fss_read_parameter_e_

/**
 * FSS Delimit Parameter enumerations.
 *
 * fss_read_delimit_mode_*_e:
 *   - auto:                   Delimit mode is automatically set.
 *   - none:                   Delimit mode is set to none.
 *   - all:                    All delimits are to be applied.
 *   - content:                Content are to have delimits applied.
 *   - content_greater:        Content at this number or higher are to have delimits applied.
 *   - content_greater_object: Objects and Content at this number or higher are to have delimits applied.
 *   - content_lesser:         Content at this number or lower are to have delimits applied.
 *   - content_lesser_object:  Objects and Content at this number or lower are to have delimits applied.
 *   - content_object:         Objects and Content are to have delimits applied
 *   - object:                 Objects arre to have delimits applied.
 */
#ifndef _di_fss_read_delimit_mode_e_
  enum {
    fss_read_delimit_mode_auto_e = 0,
    fss_read_delimit_mode_none_e,
    fss_read_delimit_mode_all_e,
    fss_read_delimit_mode_content_e,
    fss_read_delimit_mode_content_greater_e,
    fss_read_delimit_mode_content_greater_object_e,
    fss_read_delimit_mode_content_lesser_e,
    fss_read_delimit_mode_content_lesser_object_e,
    fss_read_delimit_mode_content_object_e,
    fss_read_delimit_mode_object_e,
  }; // enum
#endif // _di_fss_read_delimit_mode_e_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fss_read_common_enumeration_h
