#include "../fss_read.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_fss_read_print_at_
  f_status_t fss_read_print_at(fl_print_t * const print, const f_number_unsigned_t at, const f_number_unsigneds_t delimits_object, const f_number_unsigneds_t delimits_content) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fss_read_main_t * const main = (fss_read_main_t *) print->custom;

    if (at >= main->setting.contents.used) return F_output_not;

    if (!(main->setting.flag & fss_read_main_flag_object_d)) {
      if (!(main->setting.flag & fss_read_main_flag_content_d) || (!main->setting.contents.array[at].used && !(main->setting.flag & fss_read_main_flag_empty_d))) {
        return F_okay;
      }
    }

    // 0x1 = object printed, 0x2 = content printed, 0x4 = object end printed, 0x8 = content is empty, 0x10 = select is out of range.
    uint8_t print_state = 0x0;

    if (main->setting.flag & fss_read_main_flag_object_d) {
      if (main->callback.print_object) {
        print_state = 0x1;

        main->callback.print_object(&main->program.output, at, delimits_object);
      }
    }

    if (main->setting.flag & fss_read_main_flag_content_d) {
      if (main->setting.flag & fss_read_main_flag_select_d) {
        if (main->setting.select < main->setting.contents.array[at].used && main->setting.contents.array[at].array[main->setting.select].start <= main->setting.contents.array[at].array[main->setting.select].stop) {
          if (main->callback.print_content && main->callback.print_object_end_content) {
            if (print_state & 0x1) {
              if (F_status_set_fine(
                main->callback.print_object_end_content(
                  &main->program.output,
                  at,
                  main->setting.contents.array[at].array[main->setting.select],
                  (at < main->setting.quotes_content.used && main->setting.select < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[main->setting.select] : 0,
                  delimits_content
                )
              ) != F_output_not) {
                print_state |= 0x6;
              }
            }
            else {
              if (F_status_set_fine(
                main->callback.print_content(
                  &main->program.output,
                  at,
                  main->setting.contents.array[at].array[main->setting.select],
                  (at < main->setting.quotes_content.used && main->setting.select < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[main->setting.select] : 0,
                  delimits_content
                )
              ) != F_output_not) {
                print_state |= 0x2;
              }
            }
          }
        }
        else {
          print_state |= (main->setting.select < main->setting.contents.array[at].used) ? 0x8 : 0x18;
        }
      }
      else if (main->setting.contents.array[at].used) {
        if (main->callback.print_content && main->callback.print_object_end_content) {
          f_number_unsigned_t i = 0;

          // Loop through with conditionals only when necessary for performance reasons.
          for (print_state |= 0x8; i < main->setting.contents.array[at].used; ++i) {

            if (fss_read_signal_check(main)) return F_status_set_error(F_interrupt);

            if (main->setting.contents.array[at].array[i].start <= main->setting.contents.array[at].array[i].stop) {
              print_state &= ~0x8;

              if (print_state & 0x1) {
                if (F_status_set_fine(
                  main->callback.print_object_end_content(
                    &main->program.output,
                    at,
                    main->setting.contents.array[at].array[i],
                    (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0,
                    delimits_content
                  )
                ) != F_output_not) {
                  print_state |= 0x6;
                }
              }
              else if (F_status_set_fine(
                main->callback.print_content(
                  &main->program.output,
                  at,
                  main->setting.contents.array[at].array[i],
                  (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0,
                  delimits_content
                )
              ) != F_output_not) {
                print_state |= 0x2;
              }

              ++i;

              break;
            }
          } // for

          for (; i < main->setting.contents.array[at].used; ++i) {

            if (fss_read_signal_check(main)) return F_status_set_error(F_interrupt);
            if (main->setting.contents.array[at].array[i].start > main->setting.contents.array[at].array[i].stop) continue;

            if (main->callback.print_content_next) {
              main->callback.print_content_next(&main->program.output);
            }

            main->callback.print_content(
              &main->program.output,
              at,
              main->setting.contents.array[at].array[i],
              (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0,
              delimits_content
            );
          } // for
        }
      }
      else {
        print_state |= 0x8;
      }

      if (print_state & 0x8) {
        if (print_state & 0x1) {
          if (!(print_state & 0x4) && main->callback.print_object_end_empty) {
            main->callback.print_object_end_empty(&main->program.output);
          }
        }

        if ((print_state & 0x1) || !(print_state & 0x10)) {
          if (main->callback.print_set_end) {
            main->callback.print_set_end(&main->program.output, at);
          }
        }
      }
      else {
        if ((print_state & 0x3) != 0x3 && (print_state & 0x1)) {
          if (!(print_state & 0x4) && main->callback.print_object_end) {
            main->callback.print_object_end(&main->program.output);
          }
        }

        if (main->callback.print_set_end) {
          main->callback.print_set_end(&main->program.output, at);
        }
      }
    }
    else if (print_state & 0x1) {
      if (main->callback.print_object_end_empty) {
        main->callback.print_object_end_empty(&main->program.output);
      }

      if (main->callback.print_set_end) {
        main->callback.print_set_end(&main->program.output, at);
      }
    }

    return F_okay;
  }
#endif // _di_fss_read_print_at_

#ifndef _di_fss_read_print_content_
  f_status_t fss_read_print_content(fl_print_t * const print, const f_number_unsigned_t at, const f_range_t range, const uint8_t quote, const f_number_unsigneds_t delimits) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fss_read_main_t * const main = (fss_read_main_t *) print->custom;

    if (main->callback.print_content_ignore) {
      main->callback.print_content_ignore(&main->program.output);
    }

    if (main->setting.flag & fss_read_main_flag_original_d) {
      if (main->setting.feature & fss_read_feature_flag_quote_content_d) {
        fss_read_print_quote(&main->program.output, quote);
      }
    }

    fll_print_except_in_dynamic_partial(main->setting.buffer, range, delimits, main->setting.comments, print->to);

    if (main->setting.flag & fss_read_main_flag_original_d) {
      if (main->setting.feature & fss_read_feature_flag_quote_content_d) {
        fss_read_print_quote(&main->program.output, quote);
      }
    }

    if (main->callback.print_content_ignore) {
      main->callback.print_content_ignore(&main->program.output);
    }

    return F_okay;
  }
#endif // _di_fss_read_print_content_

#ifndef _di_fss_read_print_content_empty_
  f_status_t fss_read_print_content_empty(fl_print_t * const print) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fss_read_main_t * const main = (fss_read_main_t *) print->custom;

    if ((main->setting.flag & fss_read_main_flag_content_d) && (main->setting.flag & fss_read_main_flag_pipe_format_d)) {
      fll_print_dynamic_raw(fss_read_pipe_content_start_s, print->to);
    }
    else {
      return F_output_not;
    }

    return F_okay;
  }
#endif // _di_fss_read_print_content_empty_

#ifndef _di_fss_read_print_content_ignore_
  f_status_t fss_read_print_content_ignore(fl_print_t * const print) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    if (((fss_read_main_t *) print->custom)->setting.flag & fss_read_main_flag_pipe_format_d) {
      fll_print_dynamic_raw(fss_read_pipe_content_ignore_s, print->to);
    }
    else {
      return F_output_not;
    }

    return F_okay;
  }
#endif // _di_fss_read_print_content_ignore_

#ifndef _di_fss_read_print_number_
  f_status_t fss_read_print_number(fl_print_t * const print, const f_number_unsigned_t number) {

    if (!print) return F_status_set_error(F_output_not);

    fll_print_format("%ul%r", print->to, number, f_string_eol_s);

    return F_okay;
  }
#endif // _di_fss_read_print_number_

#ifndef _di_fss_read_print_object_
  f_status_t fss_read_print_object(fl_print_t * const print, const f_number_unsigned_t at, const f_number_unsigneds_t delimits) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fss_read_main_t * const main = (fss_read_main_t *) print->custom;

    if (at >= main->setting.objects.used) return F_output_not;

    // The current FSS standards that have a only spaces to the left of a close so simply print all to the left.
    if ((main->setting.feature & fss_read_feature_flag_object_align_d) && (main->setting.flag & fss_read_main_flag_content_d)) {
      if (!(main->setting.flag & fss_read_main_flag_original_d) || (main->setting.flag & fss_read_main_flag_trim_d)) {
        if (at < main->setting.closes.used && main->setting.closes.array[at].start <= main->setting.closes.array[at].stop) {
          f_range_t before = f_range_t_initialize;

          if (main->setting.closes.array[at].start) {
            before.start = main->setting.closes.array[at].start;
            before.stop = main->setting.closes.array[at].stop ? main->setting.closes.array[at].stop - 1 : 0;
          }

          if (before.start <= before.stop) {
            fll_print_except_dynamic_partial(main->setting.buffer, before, delimits, main->program.output.to);
          }
        }
      }
    }

    if (main->setting.flag & fss_read_main_flag_trim_d) {
      if (main->setting.flag & fss_read_main_flag_original_d) {
        if (at < main->setting.quotes_object.used && main->setting.quotes_object.array[at]) {
          fss_read_print_quote(&main->program.output, main->setting.quotes_object.array[at]);
        }

        fll_print_trim_dynamic_partial(main->setting.buffer, main->setting.objects.array[at], main->program.output.to);

        if (at < main->setting.quotes_object.used && main->setting.quotes_object.array[at]) {
          fss_read_print_quote(&main->program.output, main->setting.quotes_object.array[at]);
        }
      }
      else {
        fll_print_trim_except_dynamic_partial(main->setting.buffer, main->setting.objects.array[at], delimits, main->program.output.to);
      }
    }
    else {
      if (main->setting.flag & fss_read_main_flag_original_d) {
        if (main->setting.feature & fss_read_feature_flag_object_trim_expand_d) {
          if (main->setting.buffer.used) {
            f_range_t before = f_range_t_initialize;
            f_range_t after = f_range_t_initialize;

            if (main->setting.objects.array[at].start <= main->setting.objects.array[at].stop) {
              if (main->setting.objects.array[at].start) {
                before.start = before.stop = main->setting.objects.array[at].start - 1;

                while (before.start && main->setting.buffer.string[before.start] != f_string_eol_s.string[0]) --before.start;

                if (main->setting.buffer.string[before.start] == f_string_eol_s.string[0]) ++before.start;
              }

              if (main->setting.open_object.used && main->setting.objects.array[at].stop + 1 < main->setting.buffer.used) {
                after.start = after.stop = main->setting.objects.array[at].stop + 1;

                if (main->setting.buffer.string[after.stop] == main->setting.open_object.string[0]) {
                  after.start = 1;
                  after.stop = 0;
                }
                else {
                  while (after.stop < main->setting.buffer.used && main->setting.buffer.string[after.stop] != main->setting.open_object.string[0]) ++after.stop;

                  if (after.stop == main->setting.buffer.used || main->setting.buffer.string[after.stop] == main->setting.open_object.string[0]) --after.stop;
                }
              }
            }
            else if ((main->setting.flag & fss_read_main_flag_content_d) && (main->setting.feature & fss_read_feature_flag_object_as_line_d)) {

              // An empty Object might still have white space.
              if (at < main->setting.contents.used && main->setting.contents.array[at].used && main->setting.contents.array[at].array[0].start <= main->setting.contents.array[at].array[0].stop) {

                // Skip past the new line and the Object opener.
                if (main->setting.contents.array[at].array[0].start && main->setting.contents.array[at].array[0].start - 1) {
                  before.stop = main->setting.contents.array[at].array[0].start - 2;

                  if (main->setting.open_object.used) {
                    --before.stop;
                  }

                  before.start = before.stop;

                  while (before.start && main->setting.buffer.string[before.start] != f_string_eol_s.string[0]) --before.start;

                  if (main->setting.buffer.string[before.start] == f_string_eol_s.string[0]) ++before.start;
                }
              }
            }

            if (before.start <= before.stop) {
              fll_print_except_dynamic_partial(main->setting.buffer, before, delimits, main->program.output.to);
            }

            fll_print_except_dynamic_partial(main->setting.buffer, main->setting.objects.array[at], delimits, main->program.output.to);

            if (after.start <= after.stop) {
              fll_print_except_dynamic_partial(main->setting.buffer, after, delimits, main->program.output.to);
            }
          }
        }
        else {
          if (at < main->setting.quotes_object.used && main->setting.quotes_object.array[at]) {
            fss_read_print_quote(&main->program.output, main->setting.quotes_object.array[at]);
          }

          fll_print_dynamic_partial(main->setting.buffer, main->setting.objects.array[at], main->program.output.to);

          if (at < main->setting.quotes_object.used && main->setting.quotes_object.array[at]) {
            fss_read_print_quote(&main->program.output, main->setting.quotes_object.array[at]);
          }
        }
      }
      else {
        fll_print_except_dynamic_partial(main->setting.buffer, main->setting.objects.array[at], delimits, main->program.output.to);
      }
    }

    return F_okay;
  }
#endif // _di_fss_read_print_object_

#ifndef _di_fss_read_print_object_end_content_
  f_status_t fss_read_print_object_end_content(fl_print_t * const print, const f_number_unsigned_t at, const f_range_t range, const uint8_t quote, const f_number_unsigneds_t delimits) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fss_read_main_t * const main = (fss_read_main_t *) print->custom;

    if (main->callback.print_object_end) {
      const f_status_t status = main->callback.print_object_end(print);
      if (F_status_is_error(status) || status == F_output_not) return status;
    }

    if (main->callback.print_content) {
      return main->callback.print_content(print, at, range, quote, delimits);
    }

    return main->callback.print_object_end ? F_okay : F_output_not;
  }
#endif // _di_fss_read_print_object_end_content_

#ifndef _di_fss_read_print_object_end_empty_
  f_status_t fss_read_print_object_end_empty(fl_print_t * const print) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fss_read_main_t * const main = (fss_read_main_t *) print->custom;

    const f_status_t status_1 = (main->callback.print_object_end) ? main->callback.print_object_end(print) : F_output_not;
    const f_status_t status_2 = (main->callback.print_content_empty) ? main->callback.print_content_empty(print) : F_output_not;

    if (F_status_is_error(status_1)) return status_1;
    if (F_status_is_error(status_2)) return status_2;

    return (status_1 == F_okay || status_2 == F_okay) ? F_okay: F_output_not;
  }
#endif // _di_fss_read_print_object_end_empty_

#ifndef _di_fss_read_print_quote_
  f_status_t fss_read_print_quote(fl_print_t * const print, const uint8_t type) {

    if (!print) return F_status_set_error(F_output_not);

    if (type) {
      fll_print_dynamic_raw(
        type == f_fss_quote_type_single_e
          ? f_fss_quote_single_s
          : type == f_fss_quote_type_grave_e
            ? f_fss_quote_grave_s
            : f_fss_quote_double_s,
        print->to
      );
    }
    else {
      return F_output_not;
    }

    return F_okay;
  }
#endif // _di_fss_read_print_quote_

#ifndef _di_fss_read_print_set_end_
  f_status_t fss_read_print_set_end(fl_print_t * const print, const f_number_unsigned_t close) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    fll_print_dynamic_raw((((fss_read_main_t *) print->custom)->setting.flag & fss_read_main_flag_pipe_format_d) ? fss_read_pipe_content_end_s : f_string_eol_s, print->to);

    return F_okay;
  }
#endif // _di_fss_read_print_set_end_

#ifndef _di_fss_read_print_set_end_no_eol_
  f_status_t fss_read_print_set_end_no_eol(fl_print_t * const print, const f_number_unsigned_t close) {

    if (!print || !print->custom) return F_status_set_error(F_output_not);

    if (((fss_read_main_t *) print->custom)->setting.flag & fss_read_main_flag_pipe_format_d) {
      fll_print_dynamic_raw(fss_read_pipe_content_end_s, print->to);
    }
    else {
      return F_output_not;
    }

    return F_okay;
  }
#endif // _di_fss_read_print_set_end_no_eol_

#ifdef __cplusplus
} // extern "C"
#endif
