/**
 * FLL - Level 3
 *
 * Project: FSS Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 */
#ifndef _fss_read_payload_print_h
#define _fss_read_payload_print_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * (FSS Payload specific) Print the Content.
 *
 * This locks, uses, and unlocks the file stream.
 *
 * This processes single Content range.
 * This does not print a new line after the Content.
 *
 * @param print
 *   The output structure to print to.
 *
 *   The print.custom is expected to be of type fss_read_main_t.
 *
 *   This does not alter print.custom.setting.state.status.
 * @param at
 *   The Object index position to be processed.
 * @param range
 *   The range within the buffer representing the Content to print.
 * @param quote
 *   The quote type representing the character to use (from the f_fss_quote_type_*_e).
 * @param delimits
 *   The delimits array representing a delimited Content.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if a parameter is NULL.
 *
 * @see main.callback.print_content()
 * @see main.callback.print_object_end()
 * @see main.callback.print_set_end()
 */
#ifndef _di_fss_read_payload_print_content_
  extern f_status_t fss_read_payload_print_content(fl_print_t * const print, const f_number_unsigned_t at, const f_range_t range, const uint8_t quote, const f_number_unsigneds_t delimits);
#endif // _di_fss_read_payload_print_content_

/**
 * Print the end of an Object (which is often the start of Content).
 *
 * This locks, uses, and unlocks the file stream.
 *
 * @param print
 *   The output structure to print to.
 *
 *   The print.custom is expected to be of type fss_read_main_t.
 *
 *   This does not alter print.custom.setting.state.status.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if a parameter is NULL.
 */
#ifndef _di_fss_read_payload_print_object_end_
  extern f_status_t fss_read_payload_print_object_end(fl_print_t * const print);
#endif // _di_fss_read_payload_print_object_end_

/**
 * Print an error message about the payload parameter not having a valid value.
 *
 * @param print
 *   The output structure to print to.
 *
 *   The print.custom is expected to be of type fss_read_main_t.
 *
 *   This does not alter print.custom.setting.state.status.
 * @param value
 *   The parameter value that is invalid.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if a parameter is NULL.
 */
#ifndef _di_fss_read_payload_print_error_parameter_value_payload_
  extern f_status_t fss_read_payload_print_error_parameter_value_payload(fl_print_t * const print, const f_string_static_t value);
#endif // _di_fss_read_payload_print_error_parameter_value_payload_

/**
 * Print help for FSS-000E (Payload).
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if setting is NULL.
 */
#ifndef _di_fss_read_payload_print_message_help_
  extern f_status_t fss_read_payload_print_message_help(fl_print_t * const print);
#endif // _di_fss_read_payload_print_message_help_

/**
 * Print payload-specific help parameters details for FSS-000E (Payload).
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if setting is NULL.
 */
#ifndef _di_fss_read_payload_print_message_help_payload_
  extern f_status_t fss_read_payload_print_message_help_payload(fl_print_t * const print);
#endif // _di_fss_read_payload_print_message_help_payload_

/**
 * Print payload-specific help special parameters for FSS-000E (Payload).
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if setting is NULL.
 */
#ifndef _di_fss_read_payload_print_message_help_special_
  extern f_status_t fss_read_payload_print_message_help_special(fl_print_t * const print);
#endif // _di_fss_read_payload_print_message_help_special_

/**
 * Print error or warning message about the "payload" not being found.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if setting is NULL.
 */
#ifndef _di_fss_read_payload_print_problem_payload_missing_
  extern f_status_t fss_read_payload_print_problem_payload_missing(fl_print_t * const print);
#endif // _di_fss_read_payload_print_problem_payload_missing_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fss_read_payload_print_h
